/****************************************************************************
 *                       California Instruments RP Series                           
 *---------------------------------------------------------------------------
 *   Copyright (c) National Instruments 1998.  All Rights Reserved.         
 *---------------------------------------------------------------------------
 *                                                                          
 * Title:    CIC_RP50.h                                        
 * Purpose:  California Instruments RP Series                                       
 *           instrument driver declarations.                                
 *                                                                          
 ****************************************************************************/

#ifndef __CIC_RP50_HEADER
#define __CIC_RP50_HEADER

#include <ivi.h>
#include <ivipower.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/****************************************************************************
 *----------------- Instrument Driver Revision Information -----------------*
 ****************************************************************************/
#define CIC_RP50_MAJOR_VERSION         1     /* Instrument driver major version */
#define CIC_RP50_MINOR_VERSION         0     /* Instrument driver minor version */

/**************************************************************************** 
 *------------------------------ Useful Macros -----------------------------* 
 ****************************************************************************/

    /* Defined values for specifying the type of a waveform or transient. */
#define CIC_RP50_VAL_VOLTAGE                IVIPOWER_VAL_VOLTAGE
#define CIC_RP50_VAL_CURRENT                IVIPOWER_VAL_CURRENT

    /* Defined values for maxTime parameter to the MeasureOutput functions */
#define CIC_RP50_VAL_MAX_TIME_INFINITE      IVIPOWER_VAL_MAX_TIME_INFINITE
#define CIC_RP50_VAL_MAX_TIME_IMMEDIATE     IVIPOWER_VAL_MAX_TIME_IMMEDIATE

/**************************************************************************** 
 *---------------------------- Attribute Defines ---------------------------* 
 ****************************************************************************/

    /*- IVI Inherent Instrument Attributes ---------------------------------*/    

        /* User Options */
#define CIC_RP50_ATTR_RANGE_CHECK               IVI_ATTR_RANGE_CHECK              /* ViBoolean */
#define CIC_RP50_ATTR_QUERY_INSTR_STATUS        IVI_ATTR_QUERY_INSTR_STATUS       /* ViBoolean */
#define CIC_RP50_ATTR_CACHE                     IVI_ATTR_CACHE                    /* ViBoolean */
#define CIC_RP50_ATTR_SIMULATE                  IVI_ATTR_SIMULATE                 /* ViBoolean */
#define CIC_RP50_ATTR_RECORD_COERCIONS          IVI_ATTR_RECORD_COERCIONS         /* ViBoolean */

        /* Instrument Capabilities */
#define CIC_RP50_ATTR_NUM_CHANNELS              IVI_ATTR_NUM_CHANNELS             /* ViInt32, Read-only */
#define CIC_RP50_ATTR_SPECIFIC_PREFIX           IVI_ATTR_SPECIFIC_PREFIX          /* ViString, Read-only */

        /* Version Info */
#define CIC_RP50_ATTR_DRIVER_MAJOR_VERSION      IVI_ATTR_DRIVER_MAJOR_VERSION     /* ViInt32, Read-only */
#define CIC_RP50_ATTR_DRIVER_MINOR_VERSION      IVI_ATTR_DRIVER_MINOR_VERSION     /* ViInt32, Read-only */
#define CIC_RP50_ATTR_DRIVER_REVISION           IVI_ATTR_DRIVER_REVISION          /* ViString, Read-only */
#define CIC_RP50_ATTR_ENGINE_MAJOR_VERSION      IVI_ATTR_ENGINE_MAJOR_VERSION     /* ViInt32, Read-only */
#define CIC_RP50_ATTR_ENGINE_MINOR_VERSION      IVI_ATTR_ENGINE_MINOR_VERSION     /* ViInt32, Read-only */
#define CIC_RP50_ATTR_ENGINE_REVISION           IVI_ATTR_ENGINE_REVISION          /* ViString, Read-only */

        /* Error Info */
#define CIC_RP50_ATTR_PRIMARY_ERROR             IVI_ATTR_PRIMARY_ERROR            /* ViInt32   */
#define CIC_RP50_ATTR_SECONDARY_ERROR           IVI_ATTR_SECONDARY_ERROR          /* ViInt32   */
#define CIC_RP50_ATTR_ERROR_ELABORATION         IVI_ATTR_ERROR_ELABORATION        /* ViString  */

        /* Advanced Session I/O */
#define CIC_RP50_ATTR_VISA_RM_SESSION           IVI_ATTR_VISA_RM_SESSION          /* ViSession, Read-only */
#define CIC_RP50_ATTR_IO_SESSION                IVI_ATTR_IO_SESSION               /* ViSession, Read-only */
#define CIC_RP50_ATTR_DEFER_UPDATE              IVI_ATTR_DEFER_UPDATE             /* ViBoolean */
#define CIC_RP50_ATTR_RETURN_DEFERRED_VALUES    IVI_ATTR_RETURN_DEFERRED_VALUES   /* ViBoolean */
    
    /*- Instrument-Specific Attributes -------------------------------------*/

    /*- Basic Instrument Operation -----------------------------------------*/
#define CIC_RP50_ATTR_VOLTAGE_LIMIT             IVIPOWER_ATTR_VOLTAGE_LIMIT            /* ViReal64,  multi-channel, volts              */
#define CIC_RP50_ATTR_CURRENT_LIMIT             IVIPOWER_ATTR_CURRENT_LIMIT            /* ViReal64,  multi-channel, amperes            */
#define CIC_RP50_ATTR_OVP_ENABLED               IVIPOWER_ATTR_OVP_ENABLED              /* ViBoolean, multi-channel                     */
#define CIC_RP50_ATTR_OCP_ENABLED               IVIPOWER_ATTR_OCP_ENABLED              /* ViBoolean, multi-channel                     */
#define CIC_RP50_ATTR_OUTPUT_CONNECTED          IVIPOWER_ATTR_OUTPUT_CONNECTED         /* ViBoolean, multi-channel                     */

    /*- AC Fundamental -----------------------------------------------------*/
#define CIC_RP50_ATTR_FREQUENCY                 IVIPOWER_ATTR_FREQUENCY                /* ViReal64, multi-channel, hertz     */

    /*- AC Voltage ---------------------------------------------------------*/
#define CIC_RP50_ATTR_ACV_AMPLITUDE             IVIPOWER_ATTR_ACV_AMPLITUDE            /* ViReal64, multi-channel, volts RMS */

    /*- AC Current ---------------------------------------------------------*/
#define CIC_RP50_ATTR_ACC_AMPLITUDE             IVIPOWER_ATTR_ACC_AMPLITUDE            /* ViReal64, multi-channel, amperes RMS */

   
    /*- Single Point Output Measurements -----------------------------------*/
#define CIC_RP50_ATTR_ID_QUERY_RESPONSE      (IVI_SPECIFIC_PUBLIC_ATTR_BASE + 1L)      /* ViString (Read Only) */

#define CIC_RP50_ATTR_FREQUENCY_LOW_LIMIT   (IVI_SPECIFIC_PUBLIC_ATTR_BASE + 2L)
#define CIC_RP50_ATTR_FREQUENCY_HIGH_LIMIT  (IVI_SPECIFIC_PUBLIC_ATTR_BASE + 3L)
#define CIC_RP50_ATTR_QUERY_FUNCTION        (IVI_SPECIFIC_PUBLIC_ATTR_BASE + 4L)
#define CIC_RP50_ATTR_VOLTAGE_RANGE         (IVI_SPECIFIC_PUBLIC_ATTR_BASE + 5L)
#define CIC_RP50_ATTR_REMOTE_ON_OFF         (IVI_SPECIFIC_PUBLIC_ATTR_BASE + 6L)
#define CIC_RP50_ATTR_DISPLAY_MODE          (IVI_SPECIFIC_PUBLIC_ATTR_BASE + 7L)

/**************************************************************************** 
 *------------------------ Attribute Value Defines -------------------------* 
 ****************************************************************************/

    /*- Defined values for attribute CIC_RP50_ATTR_MEASURE_FUNCTION -*/
#define CIC_RP50_VAL_DC_VOLTS                  IVIPOWER_VAL_DC_VOLTS           /* default */
#define CIC_RP50_VAL_AC_VOLTS                  IVIPOWER_VAL_AC_VOLTS          
#define CIC_RP50_VAL_DC_CURRENT                IVIPOWER_VAL_DC_CURRENT        
#define CIC_RP50_VAL_AC_CURRENT                IVIPOWER_VAL_AC_CURRENT        
#define CIC_RP50_VAL_AC_PLUS_DC_VOLTS          IVIPOWER_VAL_AC_PLUS_DC_VOLTS  
#define CIC_RP50_VAL_AC_PLUS_DC_CURRENT        IVIPOWER_VAL_AC_PLUS_DC_CURRENT
#define CIC_RP50_VAL_FREQUENCY                 IVIPOWER_VAL_FREQUENCY         
#define CIC_RP50_VAL_DC_POWER                  IVIPOWER_VAL_DC_POWER          
#define CIC_RP50_VAL_AC_POWER                  IVIPOWER_VAL_AC_POWER  
#define CIC_RP50_VAL_PEAK_CURRENT			   10
#define CIC_RP50_VAL_CREST_FACTOR			   11  
#define CIC_RP50_VAL_APP_POWER	    		   12  
#define CIC_RP50_VAL_POWER_FACTOR			   13  
#define CIC_RP50_VAL_POWER			           14  

/*- Attribute Values generated by CVI on 8/21/98 at 8:25 AM -------------*/
#define CIC_RP50_VAL_VOLTS                                          1
#define CIC_RP50_VAL_AC_FREQUENCY                                   2
#define CIC_RP50_VAL_RANGE                                          3



/**************************************************************************** 
 *---------------- Instrument Driver Function Declarations -----------------* 
 ****************************************************************************/

    /*- Init and Close Functions -------------------------------------------*/
ViStatus _VI_FUNC CIC_RP50_init (ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice,
                                 ViSession *instrumentHandle, ViInt32 baudRate);

ViStatus _VI_FUNC CIC_RP50_InitWithOptions (ViRsrc resourceName,
                                            ViBoolean IDQuery,
                                            ViBoolean resetDevice,
                                            ViString optionString,
                                            ViSession *instrumentHandle,
                                            ViInt32 baudRate);

ViStatus _VI_FUNC  CIC_RP50_close (ViSession vi);   
ViStatus _VI_FUNC CIC_RP50_RemoteOnOff (ViSession vi, ViConstString channel,
                                        ViInt32 remoteONOFF);

ViStatus _VI_FUNC CIC_RP50_DisplayMode (ViSession vi, ViConstString channel,
                                        ViInt32 displayMode);

ViStatus _VI_FUNC CIC_RP50_DisplayModeQuery (ViSession vi, ViConstString channel,
                                             ViInt32 *displayStatus);

    /*- Basic Instrument Operation ------------------------------------------*/

ViStatus _VI_FUNC CIC_RP50_ConnectOutput (ViSession vi, ViConstString channel);

/*ViStatus _VI_FUNC CIC_RP50_DropCommands (ViSession vi,
                                         ViConstString channel,
                                         ViInt32 startphasemode,
                                         ViReal64 phaseAngle,
                                         ViReal64 outputDrop, ViReal64 voltage,
                                         ViInt16 dropMode);  */

/*ViStatus _VI_FUNC CIC_RP50_DropSettingQuery (ViSession vi, 
                                             ViConstString channelName,
                                             ViInt32 *phaseAngleStatus,
                                             ViInt32 *programmedPhaseAngle);*/

ViStatus _VI_FUNC CIC_RP50_PhaseDropCommands (ViSession vi,
                                         ViConstString channel,
                                         ViInt32 startphasemode,
                                         ViReal64 phaseAngle,
                                         ViReal64 outputDrop, 
                                         ViInt16 dropMode, ViInt16 selectPhase);

ViStatus _VI_FUNC CIC_RP50_DisconnectOutput (ViSession vi, ViConstString channel);

ViStatus _VI_FUNC CIC_RP50_PhaseDropSettingQuery (ViSession vi, 
                                             ViConstString channelName,
                                             ViInt16 selectPhase,
                                             ViInt32 *phaseAngleStatus,
                                             ViInt32 *programmedPhaseAngle);

ViStatus _VI_FUNC CIC_RP50_ConfigureLimits (ViSession vi,
                                            ViConstString channelName,
                                            ViReal64 *frequencyLowLimit,
                                            ViReal64 *frequencyHighLimit,
                                            ViReal64 *currentLimit,
                                            ViReal64 *voltageLimit,
                                            ViInt32 *modelNumber);
ViStatus _VI_FUNC CIC_RP50_GenerateACVoltageRange (ViSession vi, ViConstString channel,
                                                   ViReal64 voltageRange);
ViStatus _VI_FUNC CIC_RP50_GenerateACVoltage (ViSession vi, ViConstString channel,
                                              ViReal64 frequency);
ViStatus _VI_FUNC CIC_RP50_GenerateACCurrent (ViSession vi, ViConstString channel,
                                              ViReal64 currentLimit);
ViStatus _VI_FUNC CIC_RP50_GenerateACFrequency (ViSession vi, ViConstString channel,
                                                ViReal64 frequency);

    /*- Output Measurements ------------------------------------------------*/
ViStatus _VI_FUNC CIC_RP50_MeasureOutput (ViSession vi, ViConstString channel,
                                          ViInt32 measureFunction,
                                          ViReal64 *measurement);

ViStatus _VI_FUNC CIC_RP50_PhaseMeasureOutput (ViSession vi, ViConstString channel,
                                          ViInt32 measureFunction,
                                          ViInt16 selectPhase,
                                          ViReal64 *measurement);

    /*- Error Functions ----------------------------------------------------*/
ViStatus _VI_FUNC CIC_RP50_error_query (ViSession vi, ViInt32 *errorCode,
                                        ViChar errorMessage[]);
ViStatus _VI_FUNC CIC_RP50_GetErrorInfo (ViSession vi, ViStatus *primaryError, 
                                         ViStatus *secondaryError, 
                                         ViChar errorElaboration[256]);
ViStatus _VI_FUNC CIC_RP50_ClearErrorInfo (ViSession vi);
ViStatus _VI_FUNC CIC_RP50_StatusRegister (ViSession vi, ViInt32 *statusByte);
ViStatus _VI_FUNC CIC_RP50_error_message (ViSession vi, ViStatus errorCode,
                                          ViChar errorMessage[256]);
ViStatus _VI_FUNC CIC_RP50_QueryOutputFunction (ViSession vi, ViConstString channel,
                                                ViInt32 function, ViReal64 *value);
ViStatus _VI_FUNC CIC_RP50_QueryOutputRelay (ViSession vi, ViConstString channel,
                                             ViBoolean *status);
ViStatus _VI_FUNC CIC_RP50_SaveRegister (ViSession vi, ViConstString channel,
                                         ViInt32 registerNumber);
ViStatus _VI_FUNC CIC_RP50_RecallRegister (ViSession vi, ViConstString channel,
                                           ViInt32 registerNumber);
ViStatus _VI_FUNC CIC_RP50_PowerOnRegister (ViSession vi, ViConstString channel,
                                            ViInt32 registerNumber);
ViStatus _VI_FUNC CIC_RP50_PowerOnQuery (ViSession vi, ViConstString channel,
                                         ViInt32 *registerNumber);

ViStatus _VI_FUNC CIC_RP50_LoadInitialConfiguration (ViSession vi,
                                                     ViReal64 voltageRange,
                                                     ViReal64 frequency,
                                                     ViReal64 initialVoltage,
                                                     int outputRelay);

ViStatus _VI_FUNC CIC_RP50_LoadSerialNumber (ViSession vi,
                                             ViConstString channelName,
                                             ViChar serialNumber[]);

ViStatus _VI_FUNC CIC_RP50_Model_Number(ViSession instrumentHandle,
                                      ViChar model_number[]);

ViStatus _VI_FUNC CIC_RP50_Serial_Number(ViSession instrumentHandle,
                                      ViChar serial_number[]);

ViStatus _VI_FUNC CIC_RP50_Serial_ID (ViSession instrumentHandle,
                                      ViInt32 *serialID);

    /*- Utility Functions --------------------------------------------------*/
ViStatus _VI_FUNC CIC_RP50_reset (ViSession vi);
ViStatus _VI_FUNC CIC_RP50_revision_query (ViSession vi, 
                                           ViChar instrumentDriverRevision[],
                                           ViChar firmwareRevision[]);
ViStatus _VI_FUNC CIC_RP50_LockSession (ViSession vi, ViBoolean *callerHasLock);
ViStatus _VI_FUNC CIC_RP50_UnlockSession (ViSession vi, ViBoolean *callerHasLock);
ViStatus _VI_FUNC CIC_RP50_WriteInstrData (ViSession vi, ViConstString writeBuffer); 
ViStatus _VI_FUNC CIC_RP50_ReadInstrData  (ViSession vi, ViInt32 numBytes, 
                                           ViChar rdBuf[], ViInt32 *bytesRead);

    /*- Set, Get, and Check Attribute Functions ----------------------------*/
ViStatus _VI_FUNC CIC_RP50_GetAttributeViInt32 (ViSession vi, ViConstString channelName, ViAttr attribute, ViInt32 *value);
ViStatus _VI_FUNC CIC_RP50_GetAttributeViReal64 (ViSession vi, ViConstString channelName, ViAttr attribute, ViReal64 *value);
ViStatus _VI_FUNC CIC_RP50_GetAttributeViString (ViSession vi, ViConstString channelName, ViAttr attribute, ViInt32 bufSize, ViChar value[]); 
ViStatus _VI_FUNC CIC_RP50_GetAttributeViSession (ViSession vi, ViConstString channelName, ViAttr attribute, ViSession *value);
ViStatus _VI_FUNC CIC_RP50_GetAttributeViBoolean (ViSession vi, ViConstString channelName, ViAttr attribute, ViBoolean *value);

ViStatus _VI_FUNC CIC_RP50_SetAttributeViInt32 (ViSession vi, ViConstString channelName, ViAttr attribute, ViInt32 value);
ViStatus _VI_FUNC CIC_RP50_SetAttributeViReal64 (ViSession vi, ViConstString channelName, ViAttr attribute, ViReal64 value);
ViStatus _VI_FUNC CIC_RP50_SetAttributeViString (ViSession vi, ViConstString channelName, ViAttr attribute, ViConstString value); 
ViStatus _VI_FUNC CIC_RP50_SetAttributeViSession (ViSession vi, ViConstString channelName, ViAttr attribute, ViSession value);
ViStatus _VI_FUNC CIC_RP50_SetAttributeViBoolean (ViSession vi, ViConstString channelName, ViAttr attribute, ViBoolean value);

ViStatus _VI_FUNC CIC_RP50_CheckAttributeViInt32 (ViSession vi, ViConstString channelName, ViAttr attribute, ViInt32 value);
ViStatus _VI_FUNC CIC_RP50_CheckAttributeViReal64 (ViSession vi, ViConstString channelName, ViAttr attribute, ViReal64 value);
ViStatus _VI_FUNC CIC_RP50_CheckAttributeViString (ViSession vi, ViConstString channelName, ViAttr attribute, ViConstString value); 
ViStatus _VI_FUNC CIC_RP50_CheckAttributeViSession (ViSession vi, ViConstString channelName, ViAttr attribute, ViSession value);
ViStatus _VI_FUNC CIC_RP50_CheckAttributeViBoolean (ViSession vi, ViConstString channelName, ViAttr attribute, ViBoolean value);

    /*********************************************************
        Functions reserved for class driver use only.
        End-users should not call these functions.  
     *********************************************************/
ViStatus _VI_FUNC CIC_RP50_IviInit (ViRsrc resourceName, ViBoolean IDQuery, 
                                    ViBoolean reset, ViSession vi, ViInt32 BaudRate);
ViStatus _VI_FUNC CIC_RP50_IviClose (ViSession vi);   

/****************************************************************************
 *------------------------ Error And Completion Codes ----------------------*
 ****************************************************************************/
#define CIC_RP50_WARN_VOLTAGE_DECREASED             IVIPOWER_WARN_VOLTAGE_DECREASED
#define CIC_RP50_WARN_CURRENT_DECREASED             IVIPOWER_WARN_CURRENT_DECREASED

#define CIC_RP50_ERROR_OVER_VOLTAGE                 IVIPOWER_ERROR_OVER_VOLTAGE
#define CIC_RP50_ERROR_OVER_CURRENT                 IVIPOWER_ERROR_OVER_CURRENT
#define CIC_RP50_ERROR_OVER_TEMPERATURE             IVIPOWER_ERROR_OVER_TEMPERATURE
#define CIC_RP50_ERROR_OVER_FREQUENCY               IVI_SPECIFIC_ERROR_BASE + 1  
#define CIC_RP50_ERROR_NOT_SUPPORTED  		        IVI_SPECIFIC_ERROR_BASE + 2

/**************************************************************************** 
 *---------------------------- End Include File ----------------------------* 
 ****************************************************************************/
#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif
#endif /* __CIC_RP50_HEADER */




