#include <utility.h>
#include <gpib.h>
#include <rs232.h>
#include <ansi_c.h>
#include <formatio.h>
#include <cvirte.h>		/* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include "RPSERIES.h"
#include "CIC_RP50.h"  														

static int panelSteady, panelStart;

static ViSession RPhandle;
static ViReal64 freq_low_limit;
static ViReal64 freq_high_limit;
static ViReal64 current_limit;
static ViReal64 voltage_limit;
static ViReal64 voltagerange;
static ViReal64 currentrange;
static ViReal64 freq_value;
static ViReal64 volt_value;
static ViReal64 curr_value;
static ViReal64 rang_value;
static ViReal64 volt_query;
static ViReal64 freq_query;
static ViReal64 curr_query;
static ViReal64 volt_meas;
static ViReal64 power_meas;
static ViReal64 curr_meas;
static ViBoolean output_relay;
static ViInt32 relay, model_number;
static ViInt32 range_button;
static ViInt32 remoteonoff;
static ViInt32 measure_check;
static ViInt32 display_mode;
static ViInt32 register_status;
static int register_number;
static char resource_Name[15];
static char int_message[20], model[10];
static int address;
static int device;
static int comport;
static int serial_flag;
static int GPIBError;
static int RS232Error; 
static int port_open;
static int status, gpib_option, angle, startenable;
static ViStatus error_status;
static int mode, dropphase, startenable, angle;
double droptime;

/****************************************************************/
void DisplayGPIBError (void);
void DisplayRS232Error (void);
void Error_check (char *error_message, int error_val);
void Query_display(void);
/****************************************************************/ 

int main (int argc, char *argv[])
{
	if (InitCVIRTE (0, argv, 0) == 0)	/* Needed if linking in external compiler; harmless otherwise */
		return -1;	/* out of memory */
	
	if ((panelStart = LoadPanel (0, "RPSERIES.UIR", PANEL_STAR)) < 0)
		//return -1;
		Error_check ("Error Loading Main Window", panelStart);
	DisplayPanel (panelStart);
	RunUserInterface ();
	return 0;
}

int CVICALLBACK interfaceGPIB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{	
	//ViChar rdBuf[100];
	//ViInt32 byteRead;    
	  
	switch( event ){
		case EVENT_COMMIT:
		/* Initialize and collect all limits */
		SetCtrlVal( panelStart, PANEL_STAR_start_message, "Wait for collecting data..." );   
		//GetCtrlVal( panelStart, PANEL_STAR_gpib_button, &gpib_option );
		voltage_limit = 0;
		//if( gpib_option == 1 ){
		GetCtrlVal( panelStart, PANEL_STAR_gpib_address, &address );
		DisableBreakOnLibraryErrors();

    	device = ibdev( 0, address, NO_SAD, T10s, 1, 0 );
    	GPIBError = ibwrt( device, "*IDN?\r\n", strlen("*IDN?\r\n") );
    	if( ibsta & 0x8000 ){ 
    	  	DisplayGPIBError();
    	    SetCtrlVal( panelStart, PANEL_STAR_gpib_button, 0 );
    	    return -1;
    	}
    	else{
            Fmt( resource_Name, "%s<GPIB::%i::INSTR", address );
       	}
		/*}
		else{
			port_open = 0;
			GetCtrlVal( panelStart, PANEL_STAR_serial_port, &comport );
			GetCtrlVal( panelStart, PANEL_STAR_MODEL, model );     
			
			if( ((CompareStrings(model, 0, "2001RP", 0, 0)) == 0) || ((CompareStrings (model, 0, "2003RP", 0, 0)) == 0)){
				DisableBreakOnLibraryErrors();   
				RS232Error =  OpenComConfig( comport, "", 19200, 0, 8, 1, 512, 512 );
				port_open = 1;
			}			  
			else{
				DisableBreakOnLibraryErrors();   
				RS232Error =  OpenComConfig (comport, "", 9600, 0, 8, 1, 512, 512);
				port_open = 1;
			}				 		
			
			if( RS232Error == 0 ){
			  	if( port_open )
			  		CloseCom( comport );
			    	Fmt(resource_Name, "%s<ASRL%i::INSTR", comport); 
			     	//EnableBreakOnLibraryErrors();  
			}
			else{
			   	DisplayRS232Error(); 
			   	if( port_open )
			   		CloseCom( comport ); 
			   	    SetCtrlVal(panelStart, PANEL_STAR_serial_button, 0);
			   	  	return -1;
			}  
		}*/
		
		/* Check for communication */  
		//if( gpib_option == 1 )
		  	error_status = CIC_RP50_init (resource_Name, 1, 0, &RPhandle, 9600); 
		/*else{			
			if( ((CompareStrings(model, 0, "2001RP", 0, 0)) == 0) || ((CompareStrings (model, 0, "2003RP", 0, 0)) == 0))
				error_status = CIC_RP50_init (resource_Name, 1, 0, &RPhandle, 19200); 			
			else
				error_status = CIC_RP50_init (resource_Name, 1, 0, &RPhandle, 9600); 		
		}		
				
		if( error_status != 0 ){
			MessagePopup ("RP Series Error", "Error initializing instrument.\nPlease check your cable or your connection.");
			SetCtrlVal(panelStart, PANEL_STAR_serial_button, 0);
			SetCtrlVal(panelStart, PANEL_STAR_start_message, "Choose the address or com port before selecting GPIB or RS-232");          
			return -1;
		}*/

		EnableBreakOnLibraryErrors();     
		CIC_RP50_ConfigureLimits( RPhandle, "1", &freq_low_limit, &freq_high_limit, &current_limit, &voltage_limit, &model_number );
		Delay(0.05);
			
/*	
CIC_RP50_WriteInstrData (RPhandle, "SYST:ERR?");  
CIC_RP50_ReadInstrData (RPhandle, 20, rdBuf, &byteRead);
  			
CIC_RP50_WriteInstrData (RPhandle, "OUTP 1");
CIC_RP50_WriteInstrData (RPhandle, "OUTP?");  
CIC_RP50_ReadInstrData (RPhandle, 20, rdBuf, &byteRead);
			

CIC_RP50_WriteInstrData (RPhandle, "VOLT 108");
CIC_RP50_WriteInstrData (RPhandle, "VOLT?");  
CIC_RP50_ReadInstrData (RPhandle, 20, rdBuf, &byteRead);*/

			
		if( (panelSteady = LoadPanel(0, "RPSERIES.UIR", PANEL_ST)) < 0 )
			return -1;	  
			DisplayPanel (panelSteady); 
			break;
		
		case EVENT_RIGHT_CLICK:
		    MessagePopup ("RP Series Controller", "Choose the address or com port before selecting the interface.");
	}
	return 0;
}

int CVICALLBACK interfaceRS232 (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{	
	switch( event ){
		case EVENT_COMMIT:
		/* Initialize and collect all limits */
		SetCtrlVal( panelStart, PANEL_STAR_start_message, "Wait for collecting data..." );   
		//GetCtrlVal( panelStart, PANEL_STAR_gpib_button, &gpib_option );
		voltage_limit = 0;
		/*if( gpib_option == 1 ){
			GetCtrlVal( panelStart, PANEL_STAR_gpib_address, &address );
			DisableBreakOnLibraryErrors();

        	device = ibdev( 0, address, NO_SAD, T10s, 1, 0 );
        	GPIBError = ibwrt( device, "*IDN?\r\n", strlen("*IDN?\r\n") );
        	if( ibsta & 0x8000 ){ 
        	  	DisplayGPIBError();
        	    SetCtrlVal( panelStart, PANEL_STAR_gpib_button, 0 );
        	    return -1;
        	}
        	else{
                Fmt( resource_Name, "%s<GPIB::%i::INSTR", address );
           	}
		}
		else{*/
		port_open = 0;
		GetCtrlVal( panelStart, PANEL_STAR_serial_port, &comport );
		GetCtrlVal( panelStart, PANEL_STAR_MODEL, model );     
		
		if( ((CompareStrings(model, 0, "2001RP", 0, 0)) == 0) || ((CompareStrings (model, 0, "2003RP", 0, 0)) == 0)){
			DisableBreakOnLibraryErrors();   
			RS232Error =  OpenComConfig( comport, "", 19200, 0, 8, 1, 512, 512 );
			port_open = 1;
		}			  
		else{
			DisableBreakOnLibraryErrors();   
			RS232Error =  OpenComConfig (comport, "", 9600, 0, 8, 1, 512, 512);
			port_open = 1;
		}				 		
		
		if( RS232Error == 0 ){
		  	if( port_open )
		  		CloseCom( comport );
		    	Fmt(resource_Name, "%s<ASRL%i::INSTR", comport); 
		     	//EnableBreakOnLibraryErrors();  
		}
		else{
		   	DisplayRS232Error(); 
		   	if( port_open )
		   		CloseCom( comport ); 
		   	    SetCtrlVal(panelStart, PANEL_STAR_serial_button, 0);
		   	  	return -1;
		}  
		//}
		
		/* Check for communication */  
		//if( gpib_option == 1 )
		  //	error_status = CIC_RP50_init (resource_Name, 1, 0, &RPhandle, 9600); 
		//else{			
		if( ((CompareStrings(model, 0, "2001RP", 0, 0)) == 0) || ((CompareStrings (model, 0, "2003RP", 0, 0)) == 0))
			error_status = CIC_RP50_init (resource_Name, 1, 0, &RPhandle, 19200); 			
		else
			error_status = CIC_RP50_init (resource_Name, 1, 0, &RPhandle, 9600); 		
		//}		
				
		if( error_status != 0 ){
			MessagePopup ("RP Series Error", "Error initializing instrument.\nPlease check your cable or your connection.");
			SetCtrlVal(panelStart, PANEL_STAR_serial_button, 0);
			SetCtrlVal(panelStart, PANEL_STAR_start_message, "Choose the address or com port before selecting GPIB or RS-232");          
			return -1;
		}

		EnableBreakOnLibraryErrors();     
		CIC_RP50_ConfigureLimits( RPhandle, "1", &freq_low_limit, &freq_high_limit, &current_limit, &voltage_limit, &model_number );
		Delay(0.05);
			
		if( (panelSteady = LoadPanel(0, "RPSERIES.UIR", PANEL_ST)) < 0 )
			return -1;	  
			DisplayPanel (panelSteady); 
			break;
		
		case EVENT_RIGHT_CLICK:
		    MessagePopup ("RP Series Controller", "Choose the address or com port before selecting the interface.");
	}
	return 0;
}

int CVICALLBACK quit (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
		    if (port_open) CloseCom (comport); 
		    status = ConfirmPopup ("RP Series Controller", "Do you want to quit the program?");
			if (status) QuitUserInterface (0);
			break;
		case EVENT_RIGHT_CLICK:
		    MessagePopup ("RP Series Controller", "Quit the program.");
	}
	return 0;
}

void DisplayRS232Error (void)
{
    char ErrorMessage[200];
    char *RS232Message;
    
    if (RS232Error < 0) {   /* Bug? ComWrtByte sets rs232error if sent a byte out? */
    
       RS232Message = GetRS232ErrorString (RS232Error); 
       Fmt (ErrorMessage, "%s<RS232 error number  %i : %s", RS232Error, RS232Message);
       MessagePopup ("RS232 Message", ErrorMessage);
       }
   
}

void DisplayGPIBError (void)
{
    char ErrorMessage[200];
    char GPIBMessage[200]; 
    
    switch (iberr) {
      case 0 : 
         Fmt (GPIBMessage, "%s<Operating System error."); 
         break;
      case 1 : 
         Fmt (GPIBMessage, "%s<Function required GPIB board to be CIC.");
         break;
      case 2 : 
         Fmt (GPIBMessage, "%s<No listeners on the GPIB.");
         break;
      case 3 : 
         Fmt (GPIBMessage, "%s<GPIB board not addressed correctly.");
         break;
      case 4 : 
         Fmt (GPIBMessage, "%s<Invalid argument to function call.");
         break;
      case 5 : 
         Fmt (GPIBMessage, "%s<GPIB board not System Controller as required.");
         break;
      case 6 : 
         Fmt (GPIBMessage, "%s<I/O operation aborted (timeout).");
         break;
      case 7 : 
         Fmt (GPIBMessage, "%s<Nonexistent GPIB board.");
         break;
      case 8 : 
         Fmt (GPIBMessage, "%s<DMA error.");
         break;
      case 10 : 
         Fmt (GPIBMessage, "%s<Asynchronous I/O in progress.");
         break;
      case 11 : 
         Fmt (GPIBMessage, "%s<No capability for operation.");
         break;
      case 12 : 
         Fmt (GPIBMessage, "%s<File system error.");
         break;
      case 14 : 
         Fmt (GPIBMessage, "%s<GPIB bus error.");
         break;
      case 15 : 
         Fmt (GPIBMessage, "%s<Serial Poll status byte queue overflow.");
         break;
      case 16 : 
         Fmt (GPIBMessage, "%s<SRQ stuck in ON position.");
         break;
      case 20 : 
         Fmt (GPIBMessage, "%s<Table problem.");
         break;
      default: 
         Fmt (GPIBMessage, "%s<No error.");
         break;
         }  
    Fmt (ErrorMessage, "%s<GPIB error number  %i : %s ", iberr, GPIBMessage);
    MessagePopup ("GPIB Message", ErrorMessage);
}

void Error_check (char *error_message, int error_val)
{
   char message[50];
   int response;
   
   Fmt (message, "%s<%s. (Error = %d) Continue?", error_message, error_val);
   response = MessagePopup ("RP Series", error_message);
   exit (-1);
}

/****************** BEGINNING OF STEADY STATE CONTROL ********************/
void Query_display (void)
{
    /* Check voltage range */
		CIC_RP50_QueryOutputFunction (RPhandle, "1", CIC_RP50_VAL_RANGE, &voltagerange);
		Delay(0.05);
		if (voltagerange < voltage_limit)
	  		{
	   		SetCtrlVal(panelSteady, PANEL_ST_range_switch, 0);
	   		currentrange = current_limit;
	  		}
		else
	  		{
	   		SetCtrlVal(panelSteady, PANEL_ST_range_switch, 1); 
	   		currentrange = current_limit / 2;
	  		}

		/* Check output relay */
		CIC_RP50_QueryOutputRelay (RPhandle, "1", &output_relay);
		Delay(0.05);
		if (output_relay == 1)
	  		{
	   		SetCtrlVal(panelSteady, PANEL_ST_output_switch, 1);
	  		}
		else
	  		{
	   		SetCtrlVal(panelSteady, PANEL_ST_output_switch, 0); 
	  		}

		/* Check Remote */
		/*CIC_RP50_RemoteOnOff (RPhandle, "1", remoteonoff);
		Delay(0.05);
		if (remoteonoff == 1)
	  		{
	   		SetCtrlVal(panelSteady, PANEL_ST_remote_checkbox, 1);
	  		}
		else
	  		{
	   		SetCtrlVal(panelSteady, PANEL_ST_remote_checkbox, 0); 
	  		}  */
	  
		/* Check Display Mode */
		CIC_RP50_DisplayModeQuery (RPhandle, "1", &display_mode);
		Delay(0.05);
		if (display_mode == 1)
	  		{
	   		SetCtrlVal(panelSteady, PANEL_ST_display_switch, 1);
	  		}
		else
	  		{
	   		SetCtrlVal(panelSteady, PANEL_ST_display_switch, 0); 
	  		}
	
		/* Query all values */
		CIC_RP50_QueryOutputFunction(RPhandle, "1", CIC_RP50_VAL_VOLTS, &volt_query);
		Delay(0.05);
		CIC_RP50_QueryOutputFunction(RPhandle, "1", CIC_RP50_VAL_FREQUENCY, &freq_query);
        Delay(0.05);
        CIC_RP50_QueryOutputFunction(RPhandle, "1", CIC_RP50_VAL_CURRENT, &curr_query);		
		Delay(0.05);
	
		/* Set all slides to limit values */
		SetCtrlAttribute(panelSteady, PANEL_ST_voltage_slide, ATTR_MAX_VALUE, voltagerange);
		SetCtrlAttribute(panelSteady, PANEL_ST_current_slide, ATTR_MAX_VALUE, currentrange);
		SetCtrlAttribute(panelSteady, PANEL_ST_frequency_slide, ATTR_MAX_VALUE, freq_high_limit);
		SetCtrlVal(panelSteady, PANEL_ST_voltage_slide, volt_query);
		SetCtrlVal(panelSteady, PANEL_ST_frequency_slide, freq_query);
		SetCtrlVal(panelSteady, PANEL_ST_current_slide, curr_query);
		DisplayPanel (panelSteady);
		SetCtrlVal(panelSteady, PANEL_ST_report_message, "Ready...");
		if (gpib_option == 1)  
		   Fmt (int_message, "%s<GPIB, Address %d", address);
		else
		   Fmt (int_message, "%s<RS-232, COM port %d", comport);  
		   
  	    SetCtrlVal(panelSteady, PANEL_ST_interface_message, int_message);
}

int CVICALLBACK steady_panel (int panel, int event, void *callbackData,
		int eventData1, int eventData2)
{
	switch (event) {
	
	case EVENT_GOT_FOCUS:  

		SetCtrlVal(panelSteady, PANEL_ST_report_message, "Wait for collecting data..."); 
		Query_display();
														 
		break;
	
	case EVENT_LOST_FOCUS:
		break;
	
	case EVENT_CLOSE:
	    status = ConfirmPopup ("RP Series Controller", "Do you want to close the program?");
		if (status) 
		   {
		    if (port_open) CloseCom (comport);  
		    CIC_RP50_close (RPhandle);
		    Delay(0.05);
		    SetCtrlVal(panelStart, PANEL_STAR_serial_button, 0);  
		    SetCtrlVal(panelStart, PANEL_STAR_gpib_button, 0);  
			DiscardPanel (panelSteady);
		   }
		break;		
	}
	return 0;
}

int CVICALLBACK frequency (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (panelSteady, PANEL_ST_frequency_slide, &freq_value);
			CIC_RP50_GenerateACFrequency (RPhandle, "1", freq_value);
			Delay(0.05);
			break;
		case EVENT_RIGHT_CLICK:
		    MessagePopup ("RP Series Controller", "Change the frequency.");
	}
	return 0;
}

int CVICALLBACK voltage (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (panelSteady, PANEL_ST_voltage_slide, &volt_value);
			error_status = CIC_RP50_GenerateACVoltage (RPhandle, "1", volt_value);
			Delay(0.05);
			if (error_status != 0)
			  {
			   MessagePopup ("RP Series Error", "Error programming instrument.\nPlease check your instrument.");
			   return -1;
			  }
			break;
		case EVENT_RIGHT_CLICK:
		    MessagePopup ("RP Series Controller", "Change the voltage.");
	}
	return 0;
}

int CVICALLBACK current (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal( panelSteady, PANEL_ST_current_slide, &curr_value );
			error_status = CIC_RP50_GenerateACCurrent( RPhandle, "1", curr_value );
			Delay(0.05);
			if( error_status != 0 ){
			 	MessagePopup( "RP Series Error", "Error programming instrument.\nPlease check your instrument." );
			   	return -1;
			}
			break;
		case EVENT_RIGHT_CLICK:
		  	MessagePopup ("RP Series Controller", "Change the current limit.");
	}
	
	return 0;
}

int CVICALLBACK range (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event){
		case EVENT_COMMIT:
			GetCtrlVal( panelSteady, PANEL_ST_range_switch, &range_button );
			if( range_button == 1 ){
				error_status = CIC_RP50_GenerateACVoltageRange( RPhandle, "1", 2.0 );//voltage_limit );
				Delay(0.05);
				if( error_status != 0 ){
					MessagePopup( "RP Series Error", "Error programming instrument.\nPlease check your instrument." );
					return -1;
				}
			   	SetCtrlAttribute( panelSteady, PANEL_ST_voltage_slide, ATTR_MAX_VALUE, voltage_limit );
			   	SetCtrlAttribute( panelSteady, PANEL_ST_current_slide, ATTR_MAX_VALUE, current_limit / 2 );
		   	}	
			else{
		   		error_status = CIC_RP50_GenerateACVoltageRange( RPhandle, "1", 1.0 );//voltage_limit / 2 );
		   		Delay(0.05);
		   		if( error_status != 0 ){
		   			MessagePopup( "RP Series Error", "Error programming instrument.\nPlease check your instrument." );
		   			return -1;
		  		}
		   		SetCtrlAttribute( panelSteady, PANEL_ST_voltage_slide, ATTR_MAX_VALUE, voltage_limit / 2 );
			   	SetCtrlAttribute( panelSteady, PANEL_ST_current_slide, ATTR_MAX_VALUE, current_limit );
			}
			
			Query_display(); 
			break;
		case EVENT_RIGHT_CLICK:
		    MessagePopup( "RP Series Controller", "Change the voltage range." );
	}
	return 0;
}

int CVICALLBACK measure (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int phase;
	
	switch (event) {
		case EVENT_COMMIT:
			//GetCtrlVal( panelSteady, PANEL_ST_meas_checkbox, &measure_check );
			//if( measure_check ){
				GetCtrlVal( panelSteady, PANEL_ST_Phase, &phase );
				CIC_RP50_PhaseMeasureOutput( RPhandle, "1", CIC_RP50_VAL_AC_VOLTS, phase, &volt_meas ); 
			   	Delay(0.10);
			   	SetCtrlVal(panelSteady, PANEL_ST_meas_volt, volt_meas);
			   	
			   	GetCtrlVal( panelSteady, PANEL_ST_Phase, &phase );  
			   	CIC_RP50_PhaseMeasureOutput (RPhandle, "1", CIC_RP50_VAL_AC_CURRENT, phase, &curr_meas); 
			   	Delay(0.05);
			   	SetCtrlVal(panelSteady, PANEL_ST_meas_curr, curr_meas);
			   	
			   	GetCtrlVal( panelSteady, PANEL_ST_Phase, &phase );  
			   	CIC_RP50_PhaseMeasureOutput (RPhandle, "1", CIC_RP50_VAL_POWER, phase, &power_meas); 
			   	Delay(0.05);
			   	SetCtrlVal(panelSteady, PANEL_ST_meas_power, power_meas);
			   	SetCtrlVal (panelSteady, PANEL_ST_meas_checkbox, 0);
			//}
			break;
		case EVENT_RIGHT_CLICK:
		  	MessagePopup ("RP Series Controller", "Measure the outputs.");
			break;
	}
	return 0;
}

int CVICALLBACK OutputDrop (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal( panelSteady, PANEL_ST_DropPhase, &dropphase );  
			GetCtrlVal( panelSteady, PANEL_ST_Mode, &mode );  
			GetCtrlVal( panelSteady, PANEL_ST_DropTime, &droptime );
			GetCtrlVal( panelSteady, PANEL_ST_START, &startenable );
				
			if( startenable == 0 ){
				angle = 0;
				CIC_RP50_PhaseDropCommands( RPhandle, "1", startenable, angle, droptime, mode, dropphase ); 
		  		Delay(0.05);
			}
			else{
				CIC_RP50_PhaseDropCommands( RPhandle, "1", startenable, angle, droptime, mode, dropphase ); 
		  		Delay(0.05);				
			}
			
			SetCtrlVal( panelSteady, PANEL_ST_meas_volt, volt_meas );
		   	SetCtrlVal( panelSteady, PANEL_ST_OUTPUTDROP, 0 );
			break;
		case EVENT_RIGHT_CLICK:
		  	MessagePopup( "RP Series Controller", "Measure the outputs." );
			break;
	}
	return 0;
}

/* Check if start enable is selected */
int CVICALLBACK Start (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int check;
	
	switch( event ){
		case EVENT_COMMIT:
			GetCtrlVal( panelSteady, PANEL_ST_START, &check );
			if( check ){
				SetCtrlAttribute( panelSteady, PANEL_ST_angle1, ATTR_DIMMED, 0 );
				SetCtrlAttribute( panelSteady, PANEL_ST_angle2, ATTR_DIMMED, 0 );   	
				SetCtrlAttribute( panelSteady, PANEL_ST_angle3, ATTR_DIMMED, 0 );   	
				SetCtrlAttribute( panelSteady, PANEL_ST_angle4, ATTR_DIMMED, 0 );   	
				
				SetCtrlVal( panelSteady, PANEL_ST_angle1, 0 );    
				SetCtrlVal( panelSteady, PANEL_ST_angle2, 0 );    
				SetCtrlVal( panelSteady, PANEL_ST_angle3, 0 );    
				SetCtrlVal( panelSteady, PANEL_ST_angle4, 0 );    
				startenable = 1;
			}
			else{
				SetCtrlAttribute( panelSteady, PANEL_ST_angle1, ATTR_DIMMED, 1 );
				SetCtrlAttribute( panelSteady, PANEL_ST_angle2, ATTR_DIMMED, 1 );   	
				SetCtrlAttribute( panelSteady, PANEL_ST_angle3, ATTR_DIMMED, 1 );   	
				SetCtrlAttribute( panelSteady, PANEL_ST_angle4, ATTR_DIMMED, 1 );   	
				
				SetCtrlVal( panelSteady, PANEL_ST_angle1, 0 );    
				SetCtrlVal( panelSteady, PANEL_ST_angle2, 0 );    
				SetCtrlVal( panelSteady, PANEL_ST_angle3, 0 );    
				SetCtrlVal( panelSteady, PANEL_ST_angle4, 0 );    
				startenable = 0;	
			}
			break;
	}
	return 0;
}

int CVICALLBACK Angle1 (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch( event ){
		case EVENT_COMMIT:
			SetCtrlVal( panelSteady, PANEL_ST_angle2, 0 );
			SetCtrlVal( panelSteady, PANEL_ST_angle3, 0 );
			SetCtrlVal( panelSteady, PANEL_ST_angle4, 0 );
			angle = 0;
			break;
		}
	return 0;
}

int CVICALLBACK Angle2 (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch( event ){
		case EVENT_COMMIT:
			SetCtrlVal( panelSteady, PANEL_ST_angle1, 0 );
			SetCtrlVal( panelSteady, PANEL_ST_angle3, 0 );
			SetCtrlVal( panelSteady, PANEL_ST_angle4, 0 );
			angle = 90;
			break;
		}
	return 0;
}

int CVICALLBACK Angle3 (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch( event ){
		case EVENT_COMMIT:
			SetCtrlVal( panelSteady, PANEL_ST_angle1, 0 );
			SetCtrlVal( panelSteady, PANEL_ST_angle2, 0 );
			SetCtrlVal( panelSteady, PANEL_ST_angle4, 0 );
			angle = 180;
			break;
		}
	return 0;
}

int CVICALLBACK Angle4 (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch( event ){
		case EVENT_COMMIT:
			SetCtrlVal( panelSteady, PANEL_ST_angle1, 0 );
			SetCtrlVal( panelSteady, PANEL_ST_angle2, 0 );
			SetCtrlVal( panelSteady, PANEL_ST_angle3, 0 );
			angle = 270;
			break;
		}
	return 0;
}

int CVICALLBACK remote_on_off (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal( panelSteady, PANEL_ST_remote_checkbox, &remoteonoff );
			CIC_RP50_RemoteOnOff( RPhandle, "1", remoteonoff );
			Delay(0.05);
			break;
		case EVENT_RIGHT_CLICK:
		    MessagePopup( "RP Series Controller", "Turn the remote light On or Off" );
	}
	return 0;
}

int CVICALLBACK output_on_off (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (panelSteady, PANEL_ST_output_switch, &relay);
			if( relay == 1 ){
			   	error_status = CIC_RP50_ConnectOutput( RPhandle, "1" );
			    Delay(0.05);
			    if( error_status != 0 ){
			       	MessagePopup ("RP Series Error", "Error programming instrument.\nPlease check your instrument.");
			       	return -1;
			   	}
			}
			else{
			   	error_status = CIC_RP50_DisconnectOutput (RPhandle, "1");
			    Delay(0.05);
			    if( error_status != 0 ){
			       	MessagePopup ("RP Series Error", "Error programming instrument.\nPlease check your instrument.");
			       	return -1;
			   	}
			}
			break;
		case EVENT_RIGHT_CLICK:
		    MessagePopup( "RP Series Controller", "Close or open the relay output" );
	}
	return 0;
}

int CVICALLBACK display_on_off (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int displaymodes;
	
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal( panelSteady, PANEL_ST_display_switch, &display_mode );
			
			if( display_mode == 0 )
				displaymodes = 2;
			else if( display_mode == 1 )
				displaymodes = 1;			
			
			error_status = CIC_RP50_DisplayMode( RPhandle, "1", displaymodes );     			  
		    Delay(0.05);
		    if( error_status != 0 ){
		       	MessagePopup ("RP Series Error", "Error programming instrument.\nPlease check your instrument.");
		        return -1;
		    }
		    break;
		case EVENT_RIGHT_CLICK:
		    MessagePopup( "RP Series Controller", "Change the display to frequency or current" );
	}
	return 0;
}

int CVICALLBACK register_control (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal( panelSteady, PANEL_ST_register_number, &register_number );
			GetCtrlVal( panelSteady, PANEL_ST_register_slide, &register_status );
			if( register_status == 0 ){
			   	error_status = CIC_RP50_SaveRegister( RPhandle, "1", register_number );
			    Delay(0.05);
			    if( error_status != 0 ){
			       	MessagePopup ("RP Series Error", "Error programming instrument.\nPlease check your instrument.");
			       	return -1;
			   	}
			}
			else{
			   	error_status = CIC_RP50_RecallRegister( RPhandle, "1", register_number);  
			   	Delay(0.05);
			   	if( error_status != 0 ){
			       	MessagePopup ("RP Series Error", "Error programming instrument.\nPlease check your instrument.");
			       	return -1;
			   	}
			   	Query_display(); 
			}
			break;
		case EVENT_RIGHT_CLICK:
		    MessagePopup ("RP Series Controller", "Recall or save the register.");
	}
	return 0;
}

int CVICALLBACK close (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			status = ConfirmPopup ("RP Series Controller", "Do you want to close the program?");
			if( status ){
			   	if( port_open )
			   		CloseCom( comport );  
			   	CIC_RP50_DisconnectOutput( RPhandle, "1" ); 
			   	Delay(0.05);
			   	SetCtrlVal( panelSteady, PANEL_ST_output_switch, 0 ); 
			   	CIC_RP50_DisplayMode( RPhandle, "1", 1 );
			   	Delay(0.05);
			   	CIC_RP50_close (RPhandle);
			   	Delay(0.05);
			   	SetCtrlVal(panelStart, PANEL_STAR_serial_button, 0);  
			   	SetCtrlVal(panelStart, PANEL_STAR_gpib_button, 0);  
			   	DiscardPanel( panelSteady );
			}
			break;
		case EVENT_RIGHT_CLICK:
		    MessagePopup( "RP Series Controller", "Close the steady state output" );
	}
	return 0;
}

